
# This file was generated by 'versioneer.py' (0.20) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2021-08-22T23:29:49+0200",
 "dirty": true,
 "error": null,
 "full-revisionid": "f0b0247e6ce03ace5533fc14a75e3301578d5539",
 "version": "0.0.7"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
